require 'rubygems'
require 'sass'
require 'coffee-script'
require 'uglifier'

require 'raptest/test_error'

module Raptest
  class Asset
    
    attr_accessor :pathname
    def initialize(pathname)
      @pathname = pathname
    end
  
    def js?
      pathname =~ /\.js\b/
    end

    def css?
      pathname =~ /\.css\b/
    end

    def coffee?
      pathname =~ /\.coffee\b/
    end
  
    def scss?
      pathname =~ /\.scss\b/ 
    end
  
    def sass?
      pathname =~ /\.sass\b/
    end
  
    def erb?
      coffee? || pathname =~ /\.erb\b/     
    end

    def source
      @source ||= File.read(pathname)
    end
  
    def result
      @result
    end
  
    def test!
      reset!
      begin
        erb! if erb?

        if js?
          coffee! if coffee?
          uglify!
        elsif css?
          minify!
        end
      rescue Exception => e
        raise ::Raptest::TestError.new(@component, pathname, e.message)      
      end

    end
  
    protected
  
    def erb!
      @component = "Erb"
      view = ActionView::Base.new(ActionController::Base.view_paths, {})   
      @result    = view.render( :inline => @result )
    end
  
    def coffee!
      @component = "Coffeescript"
      @result    = CoffeeScript.compile( @result )
    end
  
    def uglify!
      @component = "UglifyJs"
      @result    = Uglifier.compile( @result )
    end
  
    def minify!
      @component = "Sass Compiler/Minifier"
      @result    = ::Sass::Engine.new(@result, {
              :syntax => :scss,
              :cache => false,
              :read_cache => false,
              :style => :compressed
            }).render
    end
  
    private
  
    def reset!
      @component = nil  
      @result    = source
    end
  
  end
end